@@ -52,11 +52,9 @@ macro_rules! pod {
     }};
 }
 
-#[derive(Clone, Copy)]
-struct DataInfo<'a> {
+struct DataInfo {
     size: usize,
     align: usize,
-    alloc: &'a dyn Allocator,
 }
 
 // 2 purposes: Prevent monomorphization as much as possible, and allow for using
@@ -96,7 +94,7 @@ where
 
     pub fn with_capacity(capacity: usize) -> Self {
         let mut s = Self::new();
-        s.raw.realloc(Self::info(&Global), capacity);
+        s.raw.realloc(&Global, capacity);
 
         return s;
     }
@@ -111,29 +109,24 @@ where
     const ALIGN: usize = core::mem::align_of::<T>();
 
     pub fn with_allocator(allocator: A) -> Self {
+        let info = DataInfo {
+            size: Self::SIZE,
+            align: Self::ALIGN,
+        };
+
         return Self {
-            raw: RawPod::new(),
+            raw: RawPod::new(info),
             allocator,
             phantom: core::marker::PhantomData,
         };
     }
 
-    #[inline(always)]
-    fn info(alloc: &dyn Allocator) -> DataInfo {
-        return DataInfo {
-            size: Self::SIZE,
-            align: Self::ALIGN,
-            alloc,
-        };
-    }
-
     #[inline(always)]
     pub fn extend_from_slice(&mut self, data: &[T]) {
         let len = data.len();
-        let info = Self::info(self.allocator.by_ref());
-        self.raw.reserve_additional(info, len);
+        self.raw.reserve_additional(&self.allocator, len);
 
-        let ptr = self.raw.ptr(info.size, self.raw.length) as *mut T;
+        let ptr = self.raw.ptr(self.raw.length) as *mut T;
         let to_space = unsafe { core::slice::from_raw_parts_mut(ptr, len) };
         to_space.copy_from_slice(data);
 
@@ -141,10 +134,9 @@ where
     }
 
     pub fn push(&mut self, t: T) {
-        let info = Self::info(self.allocator.by_ref());
-        self.raw.reserve_additional(info, 1);
+        self.raw.reserve_additional(&self.allocator, 1);
 
-        let ptr = self.raw.ptr(info.size, self.raw.length) as *mut T;
+        let ptr = self.raw.ptr(self.raw.length) as *mut T;
         self.raw.length += 1;
 
         unsafe { *ptr = t };
@@ -152,7 +144,7 @@ where
 
     pub fn leak<'b>(self) -> &'b mut [T] {
         let len = self.raw.length;
-        let ptr = self.raw.ptr(Self::SIZE, 0) as *mut T;
+        let ptr = self.raw.ptr(0) as *mut T;
 
         core::mem::forget(self);
 
@@ -164,15 +156,14 @@ where
     }
 
     pub fn insert(&mut self, i: usize, value: T) {
-        let info = Self::info(self.allocator.by_ref());
-        self.raw.reserve_additional(info, 1);
+        self.raw.reserve_additional(&self.allocator, 1);
         self.raw.length += 1;
 
-        if self.raw.copy_range(info.size, i..self.raw.length, i + 1) {
+        if self.raw.copy_range(i..self.raw.length, i + 1) {
             panic!("invalid position");
         }
 
-        let ptr = self.raw.ptr(info.size, i) as *mut T;
+        let ptr = self.raw.ptr(i) as *mut T;
         unsafe { *ptr = value };
     }
 
@@ -180,8 +171,7 @@ where
         let range = self.raw.translate_range(range);
         let len = values.len();
 
-        let info = Self::info(self.allocator.by_ref());
-        let ptr = self.raw.splice_ptr(info, range, len) as *mut T;
+        let ptr = self.raw.splice_ptr(&self.allocator, range, len) as *mut T;
         let slice = unsafe { core::slice::from_raw_parts_mut(ptr, len) };
 
         slice.copy_from_slice(values);
@@ -192,7 +182,7 @@ where
             return None;
         }
 
-        let ptr = self.raw.ptr(Self::SIZE, self.raw.length - 1) as *const T;
+        let ptr = self.raw.ptr(self.raw.length - 1) as *const T;
         self.raw.length -= 1;
 
         return Some(unsafe { *ptr });
@@ -201,18 +191,16 @@ where
     pub fn remove(&mut self, i: usize) -> T {
         let value = self[i];
 
-        let size = Self::SIZE;
-        self.raw.copy_range(size, (i + 1)..self.raw.length, i);
+        self.raw.copy_range((i + 1)..self.raw.length, i);
         self.raw.length -= 1;
 
         return value;
     }
 
     pub fn push_repeat(&mut self, t: T, repeat: usize) {
-        let info = Self::info(self.allocator.by_ref());
-        self.raw.reserve_additional(info, repeat);
+        self.raw.reserve_additional(&self.allocator, repeat);
 
-        let ptr = self.raw.ptr(info.size, self.raw.length) as *mut T;
+        let ptr = self.raw.ptr(self.raw.length) as *mut T;
         let data = unsafe { core::slice::from_raw_parts_mut(ptr, repeat) };
         data.fill(t);
 
@@ -252,20 +240,18 @@ where
 
     #[inline(always)]
     pub fn reserve(&mut self, additional: usize) {
-        let info = Self::info(self.allocator.by_ref());
-        self.raw.reserve_additional(info, additional);
+        self.raw.reserve_additional(&self.allocator, additional);
     }
 
     #[inline(always)]
     pub fn shrink_to_fit(&mut self) {
         let len = self.raw.length;
-        let info = Self::info(self.allocator.by_ref());
-        self.raw.realloc(info, len);
+        self.raw.realloc(&self.allocator, len);
     }
 
     #[inline(always)]
     pub fn raw_ptr(&self, i: usize) -> Option<*mut T> {
-        let data = self.raw.ptr(Self::SIZE, i);
+        let data = self.raw.ptr(i);
 
         return Some(data as *mut T);
     }
@@ -276,7 +262,7 @@ where
             return None;
         }
 
-        let data = self.raw.ptr(Self::SIZE, i);
+        let data = self.raw.ptr(i);
 
         return Some(unsafe { NonNull::new_unchecked(data as *mut T) });
     }
@@ -368,7 +354,7 @@ where
 {
     #[inline(always)]
     fn drop(&mut self) {
-        self.raw.realloc(Self::info(&self.allocator), 0)
+        self.raw.realloc(&self.allocator, 0)
     }
 }
 
@@ -483,15 +469,17 @@ where
 
 struct RawPod {
     data: NonNull<u8>,
+    info: DataInfo,
     length: usize,
     capacity: usize,
 }
 
 impl RawPod {
-    fn new() -> Self {
+    fn new(info: DataInfo) -> Self {
         // We use the same trick that std::vec::Vec uses
         return Self {
             data: NonNull::dangling(),
+            info,
             length: 0,
             capacity: 0,
         };
@@ -519,42 +507,42 @@ impl RawPod {
     }
 
     #[inline(always)]
-    fn ptr(&self, size: usize, i: usize) -> *mut u8 {
-        return unsafe { self.data.as_ptr().add(size * i) };
+    fn ptr(&self, i: usize) -> *mut u8 {
+        return unsafe { self.data.as_ptr().add(self.info.size * i) };
     }
 
-    fn copy_range(&mut self, size: usize, range: Range<usize>, to: usize) -> bool {
+    fn copy_range(&mut self, range: Range<usize>, to: usize) -> bool {
         let (start, end) = (range.start, range.end);
 
         if !self.range_is_valid(start, end) {
             return true;
         }
 
-        let src = self.ptr(size, start);
-        let dest = self.ptr(size, to);
+        let src = self.ptr(start);
+        let dest = self.ptr(to);
         let copy_len = end - start;
 
         // Shift everything down to fill in that spot.
-        unsafe { core::ptr::copy(src, dest, size * copy_len) };
+        unsafe { core::ptr::copy(src, dest, self.info.size * copy_len) };
 
         return false;
     }
 
     #[inline(always)]
-    fn reserve_additional(&mut self, info: DataInfo, additional: usize) {
-        return self.reserve_total(info, self.length + additional);
+    fn reserve_additional(&mut self, alloc: &dyn Allocator, additional: usize) {
+        return self.reserve_total(alloc, self.length + additional);
     }
 
-    fn reserve_total(&mut self, info: DataInfo, needed: usize) {
+    fn reserve_total(&mut self, alloc: &dyn Allocator, needed: usize) {
         if needed <= self.capacity {
             return;
         }
 
         let new_capacity = core::cmp::max(needed, self.capacity * 3 / 2);
-        self.realloc(info, new_capacity);
+        self.realloc(alloc, new_capacity);
     }
 
-    fn splice_ptr(&mut self, info: DataInfo, range: Range<usize>, len: usize) -> *mut u8 {
+    fn splice_ptr(&mut self, alloc: &dyn Allocator, range: Range<usize>, len: usize) -> *mut u8 {
         let (start, end) = (range.start, range.end);
 
         if !self.range_is_valid(start, end) {
@@ -564,16 +552,16 @@ impl RawPod {
         let copy_target = start + len;
         let range_to_copy = end..self.length;
         let final_len = copy_target + range_to_copy.len();
-        self.reserve_total(info, final_len);
+        self.reserve_total(alloc, final_len);
 
-        self.copy_range(info.size, range_to_copy, copy_target);
+        self.copy_range(range_to_copy, copy_target);
         self.length = final_len;
 
-        return self.ptr(info.size, start);
+        return self.ptr(start);
     }
 
-    fn realloc(&mut self, info: DataInfo, elem_capacity: usize) {
-        match self.try_realloc(info, elem_capacity) {
+    fn realloc(&mut self, alloc: &dyn Allocator, elem_capacity: usize) {
+        match self.try_realloc(alloc, elem_capacity) {
             Ok(()) => {}
             Err(e) => {
                 panic!("{}", e);
@@ -581,8 +569,12 @@ impl RawPod {
         }
     }
 
-    fn try_realloc(&mut self, info: DataInfo, elem_capacity: usize) -> Result<(), &'static str> {
-        let (size, align) = (info.size, info.align);
+    fn try_realloc(
+        &mut self,
+        alloc: &dyn Allocator,
+        elem_capacity: usize,
+    ) -> Result<(), &'static str> {
+        let (size, align) = (self.info.size, self.info.align);
         let get_info = move |mut data: NonNull<[u8]>| -> (NonNull<u8>, usize) {
             let data = unsafe { data.as_mut() };
             let capacity = data.len() / size;
@@ -603,7 +595,7 @@ impl RawPod {
             (prev_size, 0) => {
                 let layout =
                     Layout::from_size_align(prev_size, align).map_err(|_| "layout failure")?;
-                unsafe { info.alloc.deallocate(self.data, layout) };
+                unsafe { alloc.deallocate(self.data, layout) };
 
                 (NonNull::dangling(), elem_capacity)
             }
@@ -611,10 +603,7 @@ impl RawPod {
             (0, new_size) => {
                 let layout =
                     Layout::from_size_align(new_size, align).map_err(|_| "layout failure")?;
-                let data = info
-                    .alloc
-                    .allocate(layout)
-                    .map_err(|_| "allocation failure")?;
+                let data = alloc.allocate(layout).map_err(|_| "allocation failure")?;
 
                 get_info(data)
             }
@@ -627,9 +616,9 @@ impl RawPod {
 
                 let result = unsafe {
                     if new_size > prev_size {
-                        info.alloc.grow(self.data, prev_layout, new_layout)
+                        alloc.grow(self.data, prev_layout, new_layout)
                     } else {
-                        info.alloc.shrink(self.data, prev_layout, new_layout)
+                        alloc.shrink(self.data, prev_layout, new_layout)
                     }
                 };
 
@@ -648,10 +637,10 @@ impl RawPod {
         return Ok(());
     }
 
-    fn with_capacity(info: DataInfo, capacity: usize) -> Self {
+    fn with_capacity(info: DataInfo, alloc: &dyn Allocator, capacity: usize) -> Self {
         // We use the same trick that std::vec::Vec uses
-        let mut s = Self::new();
-        s.realloc(info, capacity);
+        let mut s = Self::new(info);
+        s.realloc(alloc, capacity);
 
         return s;
     }
